@@ -5,43 +5,72 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-from mercurial import error, hg, merge as mergemod
-
-from tortoisehg.util import hglib
-from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib, csinfo, i18n, cmdui, status, resolve
-from tortoisehg.hgqt import qscilib, thgrepo, messageentry
+from tortoisehg.util import hglib, i18n
+from tortoisehg.util.i18n import _
+from tortoisehg.hgqt import qtlib, csinfo, cmdcore, cmdui, status, resolve
+from tortoisehg.hgqt import qscilib, thgrepo, messageentry, wctxcleaner
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
+def checkrev(repo, rev):
+    op1, op2 = repo.dirstate.parents()
+    if op1 is None:
+        return _('Backout requires a parent revision')
+
+    bctx = repo[rev]
+    a = repo.changelog.ancestor(op1, bctx.node())
+    if a != bctx.node():
+        return _('Cannot backout change on a different branch')
+
+
 class BackoutDialog(QWizard):
 
-    def __init__(self, rev, repo, parent):
+    def __init__(self, repoagent, rev, parent=None):
         super(BackoutDialog, self).__init__(parent)
+        self._repoagent = repoagent
         f = self.windowFlags()
         self.setWindowFlags(f & ~Qt.WindowContextHelpButtonHint)
 
-        self.backoutrev = rev
-        self.parentbackout = False
-        self.backoutmergeparentrev = None
+        repo = repoagent.rawRepo()
+        parentbackout = repo[rev] == repo['.']
 
-        self.setWindowTitle(_('Backout - %s') % repo.displayname)
+        self.setWindowTitle(_('Backout - %s') % repoagent.displayName())
         self.setWindowIcon(qtlib.geticon('hg-revert'))
         self.setOption(QWizard.NoBackButtonOnStartPage, True)
         self.setOption(QWizard.NoBackButtonOnLastPage, True)
         self.setOption(QWizard.IndependentPages, True)
 
-        self.addPage(SummaryPage(repo, self))
-        self.addPage(BackoutPage(repo, self))
-        self.addPage(CommitPage(repo, self))
-        self.addPage(ResultPage(repo, self))
+        self.addPage(SummaryPage(repoagent, rev, parentbackout, self))
+        self.addPage(BackoutPage(repoagent, rev, parentbackout, self))
+        self.addPage(CommitPage(repoagent, rev, parentbackout, self))
+        self.addPage(ResultPage(repoagent, self))
         self.currentIdChanged.connect(self.pageChanged)
 
         self.resize(QSize(700, 489).expandedTo(self.minimumSizeHint()))
 
-        repo.repositoryChanged.connect(self.repositoryChanged)
-        repo.configChanged.connect(self.configChanged)
+        repoagent.repositoryChanged.connect(self.repositoryChanged)
+        repoagent.configChanged.connect(self.configChanged)
+
+        self._readSettings()
+
+    def _readSettings(self):
+        qs = QSettings()
+        qs.beginGroup('backout')
+        for n in ['autoadvance', 'skiplast']:
+            self.setField(n, qs.value(n, False))
+        repo = self._repoagent.rawRepo()
+        n = 'autoresolve'
+        self.setField(n, repo.ui.configbool('tortoisehg', n,
+                                            qs.value(n, True).toBool()))
+        qs.endGroup()
+
+    def _writeSettings(self):
+        qs = QSettings()
+        qs.beginGroup('backout')
+        for n in ['autoadvance', 'autoresolve', 'skiplast']:
+            qs.setValue(n, self.field(n))
+        qs.endGroup()
 
     @pyqtSlot()
     def repositoryChanged(self):
@@ -59,11 +88,19 @@ class BackoutDialog(QWizard):
         if self.currentPage().canExit():
             super(BackoutDialog, self).reject()
 
+    def done(self, r):
+        self._writeSettings()
+        super(BackoutDialog, self).done(r)
+
 
 class BasePage(QWizardPage):
-    def __init__(self, repo, parent):
+    def __init__(self, repoagent, parent):
         super(BasePage, self).__init__(parent)
-        self.repo = repo
+        self._repoagent = repoagent
+
+    @property
+    def repo(self):
+        return self._repoagent.rawRepo()
 
     def validatePage(self):
         'user pressed NEXT button, can we proceed?'
@@ -90,14 +127,11 @@ class BasePage(QWizardPage):
 
 class SummaryPage(BasePage):
 
-    def __init__(self, repo, parent):
-        super(SummaryPage, self).__init__(repo, parent)
-        self.clean = False
-        self.th = None
-
-    def initializePage(self):
-        if self.layout():
-            return
+    def __init__(self, repoagent, backoutrev, parentbackout, parent):
+        super(SummaryPage, self).__init__(repoagent, parent)
+        self._wctxcleaner = wctxcleaner.WctxCleaner(repoagent, self)
+        self._wctxcleaner.checkStarted.connect(self._onCheckStarted)
+        self._wctxcleaner.checkFinished.connect(self._onCheckFinished)
         self.setTitle(_('Prepare to backout'))
         self.setSubTitle(_('Verify backout revision and ensure your working '
                            'directory is clean.'))
@@ -106,32 +140,12 @@ class SummaryPage(BasePage):
         self.groups = qtlib.WidgetGroups()
 
         repo = self.repo
-        try:
-            bctx = repo[self.wizard().backoutrev]
-            pctx = repo['.']
-        except error.RepoLookupError:
-            qtlib.InfoMsgBox(_('Unable to backout'),
-                             _('Backout revision not found'))
-            QTimer.singleShot(0, self.wizard().close)
-            return
+        bctx = repo[backoutrev]
+        pctx = repo['.']
 
-        if pctx == bctx:
+        if parentbackout:
             lbl = _('Backing out a parent revision is a single step operation')
             self.layout().addWidget(QLabel(u'<b>%s</b>' % lbl))
-            self.wizard().parentbackout = True
-
-        op1, op2 = repo.dirstate.parents()
-        if op1 is None:
-            qtlib.InfoMsgBox(_('Unable to backout'),
-                             _('Backout requires a parent revision'))
-            QTimer.singleShot(0, self.wizard().close)
-            return
-
-        a = repo.changelog.ancestor(op1, bctx.node())
-        if a != bctx.node():
-            qtlib.InfoMsgBox(_('Unable to backout'),
-                             _('Cannot backout change on a different branch'))
-            QTimer.singleShot(0, self.wizard().close)
 
         ## backout revision
         style = csinfo.panelstyle(contents=csinfo.PANEL_DEFAULT)
@@ -158,49 +172,6 @@ class SummaryPage(BasePage):
         self.layout().addWidget(localCsInfo)
         self.localCsInfo = localCsInfo
 
-        ## Merge revision backout handling
-        if len(bctx.parents()) > 1:
-            # Show two radio buttons letting the user which merge revision
-            # parent to backout to
-            p1rev = bctx.p1().rev()
-            p2rev = bctx.p2().rev()
-
-            def setBackoutMergeParentRev(rev):
-                self.wizard().backoutmergeparentrev = rev
-
-            setBackoutMergeParentRev(p1rev)
-
-            sep = qtlib.LabeledSeparator(_('Merge parent to backout to'))
-            self.layout().addWidget(sep)
-            self.layout().addWidget(QLabel(
-                _('To backout a <b>merge</b> revision you must select which '
-                'parent to backout to '
-                '(i.e. whose changes will be <i>kept</i>)')))
-
-            self.actionFirstParent = QRadioButton(
-                _('First Parent: revision %s (%s)') \
-                % (p1rev, str(bctx.p1())), self)
-            self.actionFirstParent.setCheckable(True)
-            self.actionFirstParent.setChecked(True)
-            self.actionFirstParent.setShortcut('CTRL+1')
-            self.actionFirstParent.setToolTip(
-                _('Backout to the first parent of the merge revision'))
-            self.actionFirstParent.clicked.connect(
-                lambda: setBackoutMergeParentRev(p1rev))
-
-            self.actionSecondParent = QRadioButton(
-                _('Second Parent: revision %s (%s)')
-                % (p2rev, str(bctx.p2())), self)
-            self.actionSecondParent.setCheckable(True)
-            self.actionSecondParent.setShortcut('CTRL+2')
-            self.actionSecondParent.setToolTip(
-                _('Backout to the second parent of the merge revision'))
-            self.actionSecondParent.clicked.connect(
-                lambda: setBackoutMergeParentRev(p2rev))
-
-            self.layout().addWidget(self.actionFirstParent)
-            self.layout().addWidget(self.actionSecondParent)
-
         ## working directory status
         sep = qtlib.LabeledSeparator(_('Working directory status'))
         self.layout().addWidget(sep)
@@ -221,7 +192,7 @@ class SummaryPage(BasePage):
                  'or <a href="discard"><b>discard</b></a> changes.')
         wd_text = QLabel(text)
         wd_text.setWordWrap(True)
-        wd_text.linkActivated.connect(self.onLinkActivated)
+        wd_text.linkActivated.connect(self._wctxcleaner.runCleaner)
         self.wd_text = wd_text
         self.groups.add(wd_text, 'dirty')
         self.layout().addWidget(wd_text)
@@ -229,50 +200,41 @@ class SummaryPage(BasePage):
         ## auto-resolve
         autoresolve_chk = QCheckBox(_('Automatically resolve merge conflicts '
                                       'where possible'))
-        autoresolve_chk.setChecked(
-            repo.ui.configbool('tortoisehg', 'autoresolve', False))
         self.registerField('autoresolve', autoresolve_chk)
         self.layout().addWidget(autoresolve_chk)
-        self.autoresolve_chk = autoresolve_chk
         self.groups.set_visible(False, 'dirty')
 
     def isComplete(self):
         'should Next button be sensitive?'
-        return self.clean
+        return self._wctxcleaner.isClean()
 
     def repositoryChanged(self):
         'repository has detected a change to changelog or parents'
         pctx = self.repo['.']
         self.localCsInfo.update(pctx)
-        self.wizard().localrev = str(pctx.rev())
 
     def canExit(self):
         'can backout tool be closed?'
-        if self.th is not None and self.th.isRunning():
-            self.th.cancel()
-            self.th.wait()
+        if self._wctxcleaner.isChecking():
+            self._wctxcleaner.cancelCheck()
         return True
 
     def currentPage(self):
         self.refresh()
 
     def refresh(self):
-        if self.th is None:
-            self.th = CheckThread(self.repo, self)
-            self.th.finished.connect(self.threadFinished)
-        if self.th.isRunning():
-            return
-        self.groups.set_visible(True, 'prog')
-        self.th.start()
+        self._wctxcleaner.check()
 
     @pyqtSlot()
-    def threadFinished(self):
+    def _onCheckStarted(self):
+        self.groups.set_visible(True, 'prog')
+
+    @pyqtSlot(bool)
+    def _onCheckFinished(self, clean):
         self.groups.set_visible(False, 'prog')
-        if self.th.canceled:
+        if self._wctxcleaner.isCheckCanceled():
             return
-        dirty, parents = self.th.results
-        self.clean = not dirty
-        if dirty:
+        if not clean:
             self.groups.set_visible(True, 'dirty')
             self.wd_status.set_status(_('<b>Uncommitted local changes '
                                         'are detected</b>'), 'thg-warning')
@@ -281,86 +243,44 @@ class SummaryPage(BasePage):
             self.wd_status.set_status(_('Clean'), True)
         self.completeChanged.emit()
 
-    @pyqtSlot(QString)
-    def onLinkActivated(self, cmd):
-        cmd = hglib.fromunicode(cmd)
-        repo = self.repo
-        if cmd == 'commit':
-            from tortoisehg.hgqt import commit
-            dlg = commit.CommitDialog(repo, [], {}, self.wizard())
-            dlg.finished.connect(dlg.deleteLater)
-            dlg.exec_()
-            self.refresh()
-        elif cmd == 'shelve':
-            from tortoisehg.hgqt import shelve
-            dlg = shelve.ShelveDialog(repo, self.wizard())
-            dlg.finished.connect(dlg.deleteLater)
-            dlg.exec_()
-            self.refresh()
-        elif cmd.startswith('discard'):
-            if cmd != 'discard:noconfirm':
-                labels = [(QMessageBox.Yes, _('&Discard')),
-                          (QMessageBox.No, _('Cancel'))]
-                if not qtlib.QuestionMsgBox(_('Confirm Discard'),
-                         _('Discard outstanding changes to working directory?'),
-                         labels=labels, parent=self):
-                    return
-            def finished(ret):
-                repo.decrementBusyCount()
-                self.refresh()
-            cmdline = ['update', '--clean', '--repository', repo.root,
-                       '--rev', '.']
-            self.runner = cmdui.Runner(True, self)
-            self.runner.commandFinished.connect(finished)
-            repo.incrementBusyCount()
-            self.runner.run(cmdline)
-        elif cmd == 'view':
-            dlg = status.StatusDialog(repo, [], {}, self)
-            dlg.exec_()
-            self.refresh()
-        else:
-            raise 'unknown command: %s' % cmd
-
 
 class BackoutPage(BasePage):
-    def __init__(self, repo, parent):
-        super(BackoutPage, self).__init__(repo, parent)
+    def __init__(self, repoagent, backoutrev, parentbackout, parent):
+        super(BackoutPage, self).__init__(repoagent, parent)
+        self._backoutrev = backoutrev
+        self._parentbackout = parentbackout
         self.backoutcomplete = False
 
         self.setTitle(_('Backing out, then merging...'))
         self.setSubTitle(_('All conflicting files will be marked unresolved.'))
         self.setLayout(QVBoxLayout())
 
-        self.cmd = cmdui.Widget(True, False, self)
-        self.cmd.commandFinished.connect(self.onCommandFinished)
-        self.cmd.setShowOutput(True)
-        self.layout().addWidget(self.cmd)
+        self._cmdlog = cmdui.LogWidget(self)
+        self.layout().addWidget(self._cmdlog)
 
         self.reslabel = QLabel()
         self.reslabel.linkActivated.connect(self.onLinkActivated)
         self.reslabel.setWordWrap(True)
         self.layout().addWidget(self.reslabel)
 
-        self.autonext = QCheckBox(_('Automatically advance to next page '
-                                    'when backout and merge are complete.'))
-        checked = QSettings().value('backout/autoadvance', False).toBool()
-        self.autonext.setChecked(checked)
-        self.autonext.toggled.connect(self.tryAutoAdvance)
-        self.layout().addWidget(self.autonext)
+        autonext = QCheckBox(_('Automatically advance to next page '
+                               'when backout and merge are complete.'))
+        autonext.clicked.connect(self.tryAutoAdvance)
+        self.registerField('autoadvance', autonext)
+        self.layout().addWidget(autonext)
 
     def currentPage(self):
-        if self.wizard().parentbackout:
+        if self._parentbackout:
             self.wizard().next()
             return
-        cmdline = ['--repository', self.repo.root, 'backout']
+        cmdline = ['backout']
         tool = self.field('autoresolve').toBool() and 'merge' or 'fail'
         cmdline += ['--tool=internal:' + tool]
-        cmdline += ['--rev', str(self.wizard().backoutrev)]
-        if self.wizard().backoutmergeparentrev:
-            cmdline += ['--parent', str(self.wizard().backoutmergeparentrev)]
-        self.repo.incrementBusyCount()
-        self.cmd.core.clearOutput()
-        self.cmd.run(cmdline)
+        cmdline += ['--rev', str(self._backoutrev)]
+        self._cmdlog.clearLog()
+        sess = self._repoagent.runCommand(cmdline, self)
+        sess.commandFinished.connect(self.onCommandFinished)
+        sess.outputReceived.connect(self._cmdlog.appendLog)
 
     def isComplete(self):
         'should Next button be sensitive?'
@@ -380,36 +300,35 @@ class BackoutPage(BasePage):
             self.reslabel.setText(_('No merge conflicts, ready to commit'))
             return True
 
+    @pyqtSlot(bool)
     def tryAutoAdvance(self, checked):
         if checked and self.isComplete():
             self.wizard().next()
 
-    def cleanupPage(self):
-        QSettings().setValue('backout/autoadvance', self.autonext.isChecked())
-
+    @pyqtSlot(int)
     def onCommandFinished(self, ret):
-        self.repo.decrementBusyCount()
         if ret in (0, 1):
             self.backoutcomplete = True
-            if self.autonext.isChecked():
+            if self.field('autoadvance').toBool():
                 self.tryAutoAdvance(True)
             self.completeChanged.emit()
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def onLinkActivated(self, cmd):
         if cmd == 'resolve':
-            dlg = resolve.ResolveDialog(self.repo, self)
-            dlg.finished.connect(dlg.deleteLater)
+            dlg = resolve.ResolveDialog(self._repoagent, self)
             dlg.exec_()
-            if self.autonext.isChecked():
+            if self.field('autoadvance').toBool():
                 self.tryAutoAdvance(True)
             self.completeChanged.emit()
 
 
 class CommitPage(BasePage):
 
-    def __init__(self, repo, parent):
-        super(CommitPage, self).__init__(repo, parent)
+    def __init__(self, repoagent, backoutrev, parentbackout, parent):
+        super(CommitPage, self).__init__(repoagent, parent)
+        self._backoutrev = backoutrev
+        self._parentbackout = parentbackout
         self.commitComplete = False
 
         self.setTitle(_('Commit backout and merge results'))
@@ -417,6 +336,8 @@ class CommitPage(BasePage):
         self.setLayout(QVBoxLayout())
         self.setCommitPage(True)
 
+        repo = repoagent.rawRepo()
+
         # csinfo
         def label_func(widget, item, ctx):
             if item == 'rev':
@@ -440,7 +361,7 @@ class CommitPage(BasePage):
         def markup_func(widget, item, value):
             if item == 'rev':
                 text, rev = value
-                if self.wizard() and self.wizard().parentbackout:
+                if parentbackout:
                     return '%s (%s)' % (text, rev)
                 else:
                     return '<a href="view">%s</a> (%s)' % (text, rev)
@@ -485,10 +406,10 @@ class CommitPage(BasePage):
         self.layout().addWidget(msgEntry)
         self.msgEntry = msgEntry
 
-        self.cmd = cmdui.Widget(True, False, self)
-        self.cmd.commandFinished.connect(self.onCommandFinished)
-        self.cmd.setShowOutput(False)
-        self.layout().addWidget(self.cmd)
+        self._cmdsession = cmdcore.nullCmdSession()
+        self._cmdlog = cmdui.LogWidget(self)
+        self._cmdlog.hide()
+        self.layout().addWidget(self._cmdlog)
 
         def tryperform():
             if self.isComplete():
@@ -498,20 +419,15 @@ class CommitPage(BasePage):
         actionEnter.triggered.connect(tryperform)
         self.addAction(actionEnter)
 
-        self.skiplast = QCheckBox(_('Skip final confirmation page, '
-                                    'close after commit.'))
-        checked = QSettings().value('backout/skiplast', False).toBool()
-        self.skiplast.setChecked(checked)
-        self.layout().addWidget(self.skiplast)
+        skiplast = QCheckBox(_('Skip final confirmation page, '
+                               'close after commit.'))
+        self.registerField('skiplast', skiplast)
+        self.layout().addWidget(skiplast)
 
         def eng_toggled(checked):
             if self.isComplete():
                 oldmsg = self.msgEntry.text()
-                if self.wizard().backoutmergeparentrev:
-                    msgset = i18n.keepgettext()._(
-                        'Backed out merge changeset: ')
-                else:
-                    msgset = i18n.keepgettext()._('Backed out changeset: ')
+                msgset = i18n.keepgettext()._('Backed out changeset: ')
                 msg = checked and msgset['id'] or msgset['str']
                 if oldmsg and oldmsg != msg:
                     if not qtlib.QuestionMsgBox(_('Confirm Discard Message'),
@@ -520,8 +436,7 @@ class CommitPage(BasePage):
                         self.engChk.setChecked(not checked)
                         self.engChk.blockSignals(False)
                         return
-                self.msgEntry.setText(msg
-                                     + str(self.repo[self.wizard().backoutrev]))
+                self.msgEntry.setText(msg + str(self.repo[self._backoutrev]))
                 self.msgEntry.moveCursorToEnd()
 
         self.engChk = QCheckBox(_('Use English backout message'))
@@ -535,36 +450,19 @@ class CommitPage(BasePage):
 
     def cleanupPage(self):
         s = QSettings()
-        s.setValue('backout/skiplast', self.skiplast.isChecked())
         self.msgEntry.saveSettings(s, 'backout/message')
 
     def currentPage(self):
         engmsg = self.repo.ui.configbool('tortoisehg', 'engmsg', False)
-        mergeparentrev = self.wizard().backoutmergeparentrev
-        if mergeparentrev:
-            msgset = i18n.keepgettext()._(
-                'Backed out merge changeset: ')
-        else:
-            msgset = i18n.keepgettext()._('Backed out changeset: ')
+        msgset = i18n.keepgettext()._('Backed out changeset: ')
         msg = engmsg and msgset['id'] or msgset['str']
-        msg += str(self.repo[self.wizard().backoutrev])
-        if mergeparentrev:
-            msg += '\n\n'
-            bctx = self.repo[self.wizard().backoutrev]
-            isp1 = (bctx.p1().rev() == mergeparentrev)
-            if isp1:
-                msg += _('Backed out merge revision '
-                    'to its first parent (%s)') % str(bctx.p1())
-            else:
-                msg += _('Backed out merge revision '
-                    'to its second parent (%s)') % str(bctx.p2())
-        self.msgEntry.setText(msg)
+        self.msgEntry.setText(msg + str(self.repo[self._backoutrev]))
         self.msgEntry.moveCursorToEnd()
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def onLinkActivated(self, cmd):
         if cmd == 'view':
-            dlg = status.StatusDialog(self.repo, [], {}, self)
+            dlg = status.StatusDialog(self._repoagent, [], {}, self)
             dlg.exec_()
             self.refresh()
 
@@ -574,52 +472,51 @@ class CommitPage(BasePage):
     def validatePage(self):
         if self.commitComplete:
             # commit succeeded, repositoryChanged() called wizard().next()
-            if self.skiplast.isChecked():
+            if self.field('skiplast').toBool():
                 self.wizard().close()
             return True
-        if self.cmd.core.running():
+        if not self._cmdsession.isFinished():
             return False
 
-        user = qtlib.getCurrentUsername(self, self.repo)
+        user = hglib.tounicode(qtlib.getCurrentUsername(self, self.repo))
         if not user:
             return False
 
-        if self.wizard().parentbackout:
+        if self._parentbackout:
             self.setTitle(_('Backing out and committing...'))
             self.setSubTitle(_('Please wait while making backout.'))
-            message = hglib.fromunicode(self.msgEntry.text())
+            message = unicode(self.msgEntry.text())
             cmdline = ['backout', '--verbose', '--message', message, '--rev',
-                       str(self.wizard().backoutrev), '--user', user,
-                       '--repository', self.repo.root]
-            if self.wizard().backoutmergeparentrev:
-                cmdline += ['--parent', str(self.wizard().backoutmergeparentrev)]
+                       str(self._backoutrev), '--user', user]
         else:
             self.setTitle(_('Committing...'))
             self.setSubTitle(_('Please wait while committing merged files.'))
-            message = hglib.fromunicode(self.msgEntry.text())
+            message = unicode(self.msgEntry.text())
             cmdline = ['commit', '--verbose', '--message', message,
-                       '--repository', self.repo.root, '--user', user]
+                       '--user', user]
         commandlines = [cmdline]
         pushafter = self.repo.ui.config('tortoisehg', 'cipushafter')
         if pushafter:
-            cmd = ['push', '--repository', self.repo.root, pushafter]
+            cmd = ['push', hglib.tounicode(pushafter)]
             commandlines.append(cmd)
 
-        self.repo.incrementBusyCount()
-        self.cmd.setShowOutput(True)
-        self.cmd.run(*commandlines)
+        self._cmdlog.show()
+        sess = self._repoagent.runCommandSequence(commandlines, self)
+        self._cmdsession = sess
+        sess.commandFinished.connect(self.onCommandFinished)
+        sess.outputReceived.connect(self._cmdlog.appendLog)
         return False
 
+    @pyqtSlot(int)
     def onCommandFinished(self, ret):
-        self.repo.decrementBusyCount()
         if ret == 0:
             self.commitComplete = True
             self.wizard().next()
 
 
 class ResultPage(BasePage):
-    def __init__(self, repo, parent):
-        super(ResultPage, self).__init__(repo, parent)
+    def __init__(self, repoagent, parent):
+        super(ResultPage, self).__init__(repoagent, parent)
         self.setTitle(_('Finished'))
         self.setSubTitle(' ')
         self.setFinalPage(True)
@@ -635,39 +532,3 @@ class ResultPage(BasePage):
     def currentPage(self):
         self.bkCsInfo.update(self.repo['tip'])
         self.wizard().setOption(QWizard.NoCancelButton, True)
-
-
-class CheckThread(QThread):
-    def __init__(self, repo, parent):
-        QThread.__init__(self, parent)
-        self.repo = hg.repository(repo.ui, repo.root)
-        self.results = (False, 1)
-        self.canceled = False
-
-    def run(self):
-        self.repo.dirstate.invalidate()
-        unresolved = False
-        for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
-            if self.canceled:
-                return
-            if status == 'u':
-                unresolved = True
-                break
-        wctx = self.repo[None]
-        dirty = bool(wctx.dirty()) or unresolved
-        self.results = (dirty, len(wctx.parents()))
-
-    def cancel(self):
-        self.canceled = True
-
-
-def run(ui, *pats, **opts):
-    from tortoisehg.util import paths
-    repo = thgrepo.repository(ui, path=paths.find_root())
-    if opts.get('rev'):
-        rev = opts.get('rev')
-    elif len(pats) == 1:
-        rev = pats[0]
-    else:
-        rev = 'tip'
-    return BackoutDialog(rev, repo, None)
