@@ -7,13 +7,14 @@
 
 import os
 import difflib
+import cPickle as pickle
 import re
 
-from mercurial import util, patch
+from mercurial import util
 
-from tortoisehg.util import hglib, colormap, thread2
-from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qscilib, qtlib, blockmatcher, lexers
+from tortoisehg.util import hglib, colormap
+from tortoisehg.util.i18n import _
+from tortoisehg.hgqt import qscilib, qtlib, blockmatcher, cmdcore, lexers
 from tortoisehg.hgqt import visdiff, filedata, fileencoding
 
 from PyQt4.QtCore import *
@@ -46,15 +47,14 @@ _ChunkSelectionMarkerMask = (
 class HgFileView(QFrame):
     "file diff, content, and annotation viewer"
 
-    linkActivated = pyqtSignal(QString)
-    fileDisplayed = pyqtSignal(QString, QString)
-    showMessage = pyqtSignal(QString)
+    linkActivated = pyqtSignal(str)
+    fileDisplayed = pyqtSignal(str, str)
+    showMessage = pyqtSignal(str)
     revisionSelected = pyqtSignal(int)
     shelveToolExited = pyqtSignal()
-    newChunkList = pyqtSignal(QString, object)
     chunkSelectionChanged = pyqtSignal()
 
-    grepRequested = pyqtSignal(unicode, dict)
+    grepRequested = pyqtSignal(str, dict)
     """Emitted (pattern, opts) when user request to search changelog"""
 
     def __init__(self, repoagent, parent):
@@ -68,8 +68,6 @@ class HgFileView(QFrame):
 
         self._repoagent = repoagent
         repo = repoagent.rawRepo()
-        # TODO: replace by repoagent if setRepo(bundlerepo) can be removed
-        self.repo = repo
 
         self.topLayout = QVBoxLayout()
 
@@ -79,7 +77,7 @@ class HgFileView(QFrame):
         self.topLayout.addLayout(hbox)
 
         self.diffToolbar = QToolBar(_('Diff Toolbar'))
-        self.diffToolbar.setIconSize(QSize(16,16))
+        self.diffToolbar.setIconSize(qtlib.smallIconSize())
         self.diffToolbar.setStyleSheet(qtlib.tbstylesheet)
         hbox.addWidget(self.diffToolbar)
 
@@ -131,10 +129,11 @@ class HgFileView(QFrame):
         self.sci.setMarginType(0, qsci.SymbolMargin)
         self.sci.setMarginWidth(0, 0)
 
-        self.searchbar = qscilib.SearchToolBar(hidable=True)
+        self.searchbar = qscilib.SearchToolBar()
         self.searchbar.hide()
         self.searchbar.searchRequested.connect(self.find)
         self.searchbar.conditionChanged.connect(self.highlightText)
+        self.addActions(self.searchbar.editorActions())
         self.layout().addWidget(self.searchbar)
 
         self._fd = self._nullfd = filedata.createNullData(repo)
@@ -165,7 +164,7 @@ class HgFileView(QFrame):
         filec.chunkMarkersBuilt.connect(self._updateDiffActions)
         messagec = _MessageViewControl(self.sci, self)
         messagec.forceDisplayRequested.connect(self._forceDisplayFile)
-        annotatec = _AnnotateViewControl(repo.ui, self.sci, self._fd, self)
+        annotatec = _AnnotateViewControl(repoagent, self.sci, self._fd, self)
         annotatec.showMessage.connect(self.showMessage)
         annotatec.editSelectedRequested.connect(self._editSelected)
         annotatec.grepRequested.connect(self.grepRequested)
@@ -173,7 +172,7 @@ class HgFileView(QFrame):
         annotatec.setSourceRequested.connect(self._setSource)
         annotatec.visualDiffRevisionRequested.connect(self._visualDiffRevision)
         annotatec.visualDiffToLocalRequested.connect(self._visualDiffToLocal)
-        chunkselc = _ChunkSelectionViewControl(self.sci, self)
+        chunkselc = _ChunkSelectionViewControl(self.sci, self._fd, self)
         chunkselc.chunkSelectionChanged.connect(self.chunkSelectionChanged)
 
         self._activeViewControls = []
@@ -212,7 +211,7 @@ class HgFileView(QFrame):
                                     self._showSearchbar)
 
         self.actionShelf = QAction('Shelve', self)
-        self.actionShelf.setIcon(qtlib.geticon('shelve'))
+        self.actionShelf.setIcon(qtlib.geticon('hg-shelve'))
         self.actionShelf.setToolTip(_('Open shelve tool'))
         self.actionShelf.setVisible(False)
         self.actionShelf.triggered.connect(self._launchShelve)
@@ -240,6 +239,10 @@ class HgFileView(QFrame):
         repoagent.configChanged.connect(self._applyRepoConfig)
         self._applyRepoConfig()
 
+    @property
+    def repo(self):
+        return self._repoagent.rawRepo()
+
     @pyqtSlot()
     def _launchShelve(self):
         from tortoisehg.hgqt import shelve
@@ -278,10 +281,6 @@ class HgFileView(QFrame):
         enc = fileencoding.contentencoding(self.repo.ui, self._textEncoding())
         self._changeTextEncoding(enc)
 
-    def setRepo(self, repo):
-        self.repo = repo
-        self.sci.repo = repo
-
     def isChangeSelectionEnabled(self):
         chunkselc = self._chunkSelectionViewControl
         controls = self._modeViewControlsMap[DiffMode]
@@ -302,19 +301,12 @@ class HgFileView(QFrame):
         if self._effectiveMode() == DiffMode:
             self._changeEffectiveMode(DiffMode)
 
-    def updateChunk(self, chunk, exclude):
-        'change chunk exclusion state, update display when necessary'
-        self._chunkSelectionViewControl.updateChunk(chunk, exclude)
-
     @pyqtSlot(QAction)
     def _setModeByAction(self, action):
         'One of the mode toolbar buttons has been toggled'
         mode = action.data().toInt()[0]
         self._lostMode = _NullMode
         self._changeEffectiveMode(mode)
-        # TODO: delete fd.load(), which is necessary because ChunkSelection
-        # view cannot start with chunk.excluded = True
-        self._fd.load(self.isChangeSelectionEnabled())
         self._displayLoaded(self._fd)
 
     def _effectiveMode(self):
@@ -367,20 +359,12 @@ class HgFileView(QFrame):
         """Switch view to DiffMode/FileMode/AnnMode if available for the current
         content; otherwise it will be switched later"""
         action = self._modeAction(mode)
-        if not action.isVisible():
-            raise ValueError('unsupported mode: %r' % mode)
-
         if action.isEnabled():
             if not action.isChecked():
                 action.trigger()  # implies _setModeByAction()
         else:
             self._lostMode = mode
 
-    def setModeVisible(self, mode, visible):
-        action = self._modeAction(mode)
-        action.setVisible(visible)
-        self._fallBackToAvailableMode()
-
     @pyqtSlot(QAction)
     def _setParentRevision(self, action):
         fd = self._fd
@@ -420,7 +404,7 @@ class HgFileView(QFrame):
         if not self._fd.isNull():
             self._applyTextEncoding()
 
-    @pyqtSlot(unicode, int, int)
+    @pyqtSlot(str, int, int)
     def _setSource(self, path, rev, line):
         # BUG: not work for subrepo
         self.revisionSelected.emit(rev)
@@ -465,7 +449,8 @@ class HgFileView(QFrame):
         self._displayLoaded(self._fd)
 
     def display(self, fd):
-        fd.load(self.isChangeSelectionEnabled())
+        if not fd.isLoaded():
+            fd.load(self.isChangeSelectionEnabled())
         fd.setTextEncoding(self._textEncoding())
         if self._autoTextEncoding():
             fd.detectTextEncoding()
@@ -510,7 +495,7 @@ class HgFileView(QFrame):
                 availablemodes.add(DiffMode)
             if fd.contents:
                 availablemodes.add(FileMode)
-            if (fd.contents and fd.rev() is not None and fd.rev() >= 0
+            if (fd.contents and (fd.rev() is None or fd.rev() >= 0)
                 and fd.fileStatus() != 'R'):
                 availablemodes.add(AnnMode)
         self._restrictModes(availablemodes)
@@ -518,11 +503,6 @@ class HgFileView(QFrame):
         for c in self._activeViewControls:
             c.display(fd)
 
-        if self._effectiveMode() == DiffMode and fd.isValid():
-            self.newChunkList.emit(fd.filePath(), fd.changes)
-        else:
-            self.newChunkList.emit(fd.filePath(), None)
-
         self.highlightText(*self._lastSearch)
         self.fileDisplayed.emit(fd.filePath(), fd.fileText())
 
@@ -548,11 +528,11 @@ class HgFileView(QFrame):
             self.maxWidth += fm.width(longestline)
         self._updateScrollBar()
 
-    @pyqtSlot(unicode, bool, bool, bool)
+    @pyqtSlot(str, bool, bool, bool)
     def find(self, exp, icase=True, wrap=False, forward=True):
         self.sci.find(exp, icase, wrap, forward)
 
-    @pyqtSlot(unicode, bool)
+    @pyqtSlot(str, bool)
     def highlightText(self, match, icase=False):
         self._lastSearch = match, icase
         self.sci.highlightText(match, icase)
@@ -615,7 +595,7 @@ class HgFileView(QFrame):
         self.actionNextDiff.setEnabled(self._findNextChunk() >= 0)
         self.actionPrevDiff.setEnabled(self._findPrevChunk() >= 0)
 
-    @pyqtSlot(unicode, int, int)
+    @pyqtSlot(str, int, int)
     def _editSelected(self, path, rev, line):
         """Open editor to show the specified file"""
         path = hglib.fromunicode(path)
@@ -630,11 +610,11 @@ class HgFileView(QFrame):
         if dlg:
             dlg.exec_()
 
-    @pyqtSlot(unicode, int)
+    @pyqtSlot(str, int)
     def _visualDiffRevision(self, path, rev):
         self._visualDiff(path, change=rev)
 
-    @pyqtSlot(unicode, int)
+    @pyqtSlot(str, int)
     def _visualDiffToLocal(self, path, rev):
         self._visualDiff(path, rev=[str(rev)])
 
@@ -923,17 +903,19 @@ class _MessageViewControl(_AbstractViewControl):
 class _AnnotateViewControl(_AbstractViewControl):
     """Display annotation margin and colorize file content in HgFileView"""
 
-    showMessage = pyqtSignal(QString)
+    showMessage = pyqtSignal(str)
 
-    editSelectedRequested = pyqtSignal(unicode, int, int)
-    grepRequested = pyqtSignal(unicode, dict)
+    editSelectedRequested = pyqtSignal(str, int, int)
+    grepRequested = pyqtSignal(str, dict)
     searchSelectedTextRequested = pyqtSignal()
-    setSourceRequested = pyqtSignal(unicode, int, int)
-    visualDiffRevisionRequested = pyqtSignal(unicode, int)
-    visualDiffToLocalRequested = pyqtSignal(unicode, int)
+    setSourceRequested = pyqtSignal(str, int, int)
+    visualDiffRevisionRequested = pyqtSignal(str, int)
+    visualDiffToLocalRequested = pyqtSignal(str, int)
 
-    def __init__(self, ui, sci, fd, parent=None):
+    def __init__(self, repoagent, sci, fd, parent=None):
         super(_AnnotateViewControl, self).__init__(parent)
+        self._repoagent = repoagent
+        self._cmdsession = cmdcore.nullCmdSession()
         self._sci = sci
         self._sci.setMarginType(_AnnotateMargin, qsci.TextMarginRightJustified)
         self._sci.setMarginSensitivity(_AnnotateMargin, True)
@@ -942,33 +924,22 @@ class _AnnotateViewControl(_AbstractViewControl):
         self._fd = fd
         self._links = []  # by line
         self._revmarkers = {}  # by rev
-        self._lastrev = None
+        self._lastrev = -1
 
         self._lastmarginclick = QTime.currentTime()
         self._lastmarginclick.addMSecs(-QApplication.doubleClickInterval())
 
-        diffopts = patch.diffopts(ui, section='annotate')
-        self._thread = AnnotateThread(self, diffopts=diffopts)
-        self._thread.finished.connect(self.fillModel)
-
-        self._abortshortcut = a = QShortcut(Qt.Key_Escape, sci)
-        a.setContext(Qt.WidgetShortcut)
-        a.setEnabled(False)
-        a.activated.connect(self._thread.abort)
-
         self._initAnnotateOptionActions()
         self._loadAnnotateSettings()
 
     def open(self):
         self._sci.viewport().installEventFilter(self)
-        self._abortshortcut.setEnabled(True)
 
     def close(self):
         self._sci.viewport().removeEventFilter(self)
         self._sci.setMarginWidth(_AnnotateMargin, 0)
         self._sci.markerDeleteAll()
-        self._abortshortcut.setEnabled(False)
-        self._thread.abort()
+        self._cmdsession.abort()
 
     def eventFilter(self, watched, event):
         # Python wrapper is deleted immediately before QEvent.Destroy
@@ -988,7 +959,7 @@ class _AnnotateViewControl(_AbstractViewControl):
         wb = "Annotate/"
         for a in self._annoptactions:
             a.setChecked(s.value(wb + a.data().toString()).toBool())
-        if not util.any(a.isChecked() for a in self._annoptactions):
+        if not any(a.isChecked() for a in self._annoptactions):
             self._annoptactions[-1].setChecked(True)  # 'rev' by default
 
     def _saveAnnotateSettings(self):
@@ -1010,7 +981,7 @@ class _AnnotateViewControl(_AbstractViewControl):
     @pyqtSlot()
     def _updateAnnotateOption(self):
         # make sure at least one option is checked
-        if not util.any(a.isChecked() for a in self._annoptactions):
+        if not any(a.isChecked() for a in self._annoptactions):
             self.sender().setChecked(True)
 
         self._updateView()
@@ -1021,62 +992,84 @@ class _AnnotateViewControl(_AbstractViewControl):
             return hglib.tounicode(hglib.username(fctx.user()))
         def getdate(fctx):
             return util.shortdate(fctx.date())
-        def getrev(fctx):
-            return fctx.rev()
+        if self._fd.rev() is None:
+            p1rev = self._fd.parentRevs()[0]
+            revfmt = '%%%dd%%c' % len(str(p1rev))
+            def getrev(fctx):
+                if fctx.rev() is None:
+                    return revfmt % (p1rev, '+')
+                else:
+                    return revfmt % (fctx.rev(), ' ')
+        else:
+            revfmt = '%%%dd' % len(str(self._fd.rev()))
+            def getrev(fctx):
+                return revfmt % fctx.rev()
 
         aformat = [str(a.data().toString()) for a in self._annoptactions
                    if a.isChecked()]
-        revwidth = len(str(self._fd.rev()))
         annfields = {
-            'rev': ('%%%dd' % revwidth, getrev),
-            'author': ('%s', getauthor),
-            'date': ('%s', getdate),
+            'rev': getrev,
+            'author': getauthor,
+            'date': getdate,
         }
-        annformat = []
-        annfunc = []
-        for fieldname in aformat:
-            s, f = annfields[fieldname]
-            annformat.append(s)
-            annfunc.append(f)
-        annformat = ' : '.join(annformat)
+        annfunc = [annfields[n] for n in aformat]
 
         uniqfctxs = set(fctx for fctx, _origline in self._links)
-        return dict((fctx.rev(), annformat % tuple(f(fctx) for f in annfunc))
+        return dict((fctx.rev(), ' : '.join(f(fctx) for f in annfunc))
                     for fctx in uniqfctxs)
 
     def _emitRevisionHintAtLine(self, line):
-        if line < 0:
+        if line < 0 or line >= len(self._links):
             return
-        try:
-            fctx = self._links[line][0]
-            if fctx.rev() != self._lastrev:
-                filename = hglib.fromunicode(self._fd.canonicalFilePath())
-                s = hglib.get_revision_desc(fctx, filename)
-                self.showMessage.emit(s)
-                self._lastrev = fctx.rev()
-        except IndexError:
-            pass
+        fctx = self._links[line][0]
+        if fctx.rev() != self._lastrev:
+            filename = hglib.fromunicode(self._fd.canonicalFilePath())
+            s = hglib.get_revision_desc(fctx, filename)
+            self.showMessage.emit(s)
+            self._lastrev = fctx.rev()
+
+    def _repoAgentForFile(self):
+        rpath = self._fd.repoRootPath()
+        if not rpath:
+            return self._repoagent
+        return self._repoagent.subRepoAgent(rpath)
 
     def display(self, fd):
-        if fd.rev() is None:
-            return
         if self._fd == fd and self._links:
             self._updateView()
             return
         self._fd = fd
         del self._links[:]
-        ctx = fd.rawContext()
-        filename = hglib.fromunicode(fd.canonicalFilePath())
-        self._thread.abort()
-        self._thread.start(ctx[filename])
+        self._cmdsession.abort()
+        repoagent = self._repoAgentForFile()
+        cmdline = hglib.buildcmdargs('annotate', fd.canonicalFilePath(),
+                                     rev=hglib.escaperev(fd.rev(), 'wdir()'),
+                                     text=True, file=True,
+                                     number=True, line_number=True, T='pickle')
+        self._cmdsession = sess = repoagent.runCommand(cmdline, self)
+        sess.setCaptureOutput(True)
+        sess.commandFinished.connect(self._onAnnotateFinished)
 
-    @pyqtSlot()
-    def fillModel(self):
-        self._thread.wait()
-        if self._thread.data is None:
+    @pyqtSlot(int)
+    def _onAnnotateFinished(self, ret):
+        sess = self._cmdsession
+        if not sess.isFinished():
+            # new request is already running
             return
-
-        self._links = list(self._thread.data)
+        if ret != 0:
+            return
+        repo = self._repoAgentForFile().rawRepo()
+        data = pickle.loads(str(sess.readAll()))
+        links = []
+        fctxcache = {}  # (path, rev): fctx
+        for l in data:
+            path, rev = l['file'], l['rev']
+            try:
+                fctx = fctxcache[path, rev]
+            except KeyError:
+                fctx = fctxcache[path, rev] = repo[rev][path]
+            links.append((fctx, l['line_number']))
+        self._links = links
         self._updateView()
 
     def _updateView(self):
@@ -1181,7 +1174,6 @@ class _AnnotateViewControl(_AbstractViewControl):
             self.visualDiffRevisionRequested.emit(data[0], data[1])
 
         menu.addSeparator()
-        origrev = fctx.rev()
         anngotomenu = menu.addMenu(_('Go to'))
         annviewmenu = menu.addMenu(_('View File at'))
         anndiffmenu = menu.addMenu(_('Diff File to'))
@@ -1215,6 +1207,9 @@ class _AnnotateViewControl(_AbstractViewControl):
         lastclick = self._lastmarginclick
         if (state == Qt.ControlModifier
             or lastclick.elapsed() < QApplication.doubleClickInterval()):
+            if line >= len(self._links):
+                # empty line next to the last line
+                return
             fctx, line = self._links[line]
             self.setSourceRequested.emit(
                 hglib.tounicode(fctx.path()), fctx.rev(), line)
@@ -1240,53 +1235,12 @@ class _AnnotateViewControl(_AbstractViewControl):
             self._sci.setSelection(sline, 0, eline, 0)
 
 
-class AnnotateThread(QThread):
-    'Background thread for annotating a file at a revision'
-    def __init__(self, parent=None, diffopts=None):
-        super(AnnotateThread, self).__init__(parent)
-        self._threadid = None
-        self._diffopts = diffopts
-
-    @pyqtSlot(object)
-    def start(self, fctx):
-        self._fctx = fctx
-        super(AnnotateThread, self).start()
-        self.data = None
-
-    @pyqtSlot()
-    def abort(self):
-        threadid = self._threadid
-        if threadid is None:
-            return
-        try:
-            thread2._async_raise(threadid, KeyboardInterrupt)
-            self.wait()
-        except ValueError:
-            pass
-
-    def run(self):
-        assert self.currentThread() != qApp.thread()
-        self._threadid = self.currentThreadId()
-        try:
-            try:
-                data = []
-                for (fctx, line), _text in \
-                        self._fctx.annotate(True, True, self._diffopts):
-                    data.append((fctx, line))
-                self.data = data
-            except KeyboardInterrupt:
-                pass
-        finally:
-            self._threadid = None
-            del self._fctx
-
-
 class _ChunkSelectionViewControl(_AbstractViewControl):
     """Display chunk selection margin and colorize chunks in HgFileView"""
 
     chunkSelectionChanged = pyqtSignal()
 
-    def __init__(self, sci, parent=None):
+    def __init__(self, sci, fd, parent=None):
         super(_ChunkSelectionViewControl, self).__init__(parent)
         self._sci = sci
         p = qtlib.getcheckboxpixmap(QStyle.State_On, QColor('#B0FFA0'), sci)
@@ -1320,7 +1274,7 @@ class _ChunkSelectionViewControl(_AbstractViewControl):
         a.setEnabled(False)
         a.activated.connect(self._toggleCurrentChunk)
 
-        self._changes = None
+        self._fd = fd
         self._chunkatline = {}
 
     def open(self):
@@ -1332,23 +1286,23 @@ class _ChunkSelectionViewControl(_AbstractViewControl):
         self._toggleshortcut.setEnabled(False)
 
     def display(self, fd):
-        self._changes = None
+        self._fd = fd
         self._chunkatline.clear()
         if not fd.changes:
             return
-        self._changes = fd.changes
         for chunk in fd.changes.hunks:
             self._chunkatline[chunk.lineno] = chunk
-            self._sci.markerAdd(chunk.lineno, _IncludedChunkStartMarker)
+            self._updateMarker(chunk)
 
-    def updateChunk(self, chunk, exclude):
-        if chunk.excluded == exclude:
-            return
+    def _updateMarker(self, chunk):
         excludemsg = ' ' + _('(excluded from the next commit)')
-        if exclude:
-            chunk.excluded = True
-            self._changes.excludecount += 1
+        # markerAdd() does not check if the specified marker is already
+        # present, but markerDelete() does
+        m = self._sci.markersAtLine(chunk.lineno)
+        inclmarked = m & (1 << _IncludedChunkStartMarker)
+        exclmarked = m & (1 << _ExcludedChunkStartMarker)
 
+        if chunk.excluded and not exclmarked:
             self._sci.setReadOnly(False)
             llen = self._sci.lineLength(chunk.lineno)  # in bytes
             self._sci.insertAt(excludemsg, chunk.lineno, llen - 1)
@@ -1361,10 +1315,8 @@ class _ChunkSelectionViewControl(_AbstractViewControl):
             self._sci.fillIndicatorRange(chunk.lineno + 1, 0,
                                          chunk.lineno + chunk.linecount, 0,
                                          self._excludeindicator)
-        else:
-            chunk.excluded = False
-            self._changes.excludecount -= 1
 
+        if not chunk.excluded and exclmarked:
             self._sci.setReadOnly(False)
             llen = self._sci.lineLength(chunk.lineno)  # in bytes
             mlen = len(excludemsg.encode('utf-8'))  # in bytes
@@ -1374,6 +1326,7 @@ class _ChunkSelectionViewControl(_AbstractViewControl):
             self._sci.SendScintilla(qsci.SCI_REPLACETARGET, 0, '')
             self._sci.setReadOnly(True)
 
+        if not chunk.excluded and not inclmarked:
             self._sci.markerDelete(chunk.lineno, _ExcludedChunkStartMarker)
             self._sci.markerAdd(chunk.lineno, _IncludedChunkStartMarker)
             for i in xrange(chunk.linecount - 1):
@@ -1411,7 +1364,9 @@ class _ChunkSelectionViewControl(_AbstractViewControl):
 
     def _setChunkAtLines(self, lines, excluded):
         for l in lines:
-            self.updateChunk(self._chunkatline[l], excluded)
+            chunk = self._chunkatline[l]
+            self._fd.setChunkExcluded(chunk, excluded)
+            self._updateMarker(chunk)
         self.chunkSelectionChanged.emit()
 
     def _toggleChunkAtLine(self, line):
