@@ -6,12 +6,10 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-from mercurial import hg, error, util
-
 from tortoisehg.util import hglib
-from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib, csinfo, i18n, cmdui, status, resolve
-from tortoisehg.hgqt import qscilib, thgrepo, messageentry, commit
+from tortoisehg.util.i18n import _
+from tortoisehg.hgqt import qtlib, csinfo, cmdcore, cmdui, status, resolve
+from tortoisehg.hgqt import qscilib, thgrepo, messageentry, commit, wctxcleaner
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
@@ -20,26 +18,23 @@ MARGINS = (8, 0, 0, 0)
 
 class MergeDialog(QWizard):
 
-    def __init__(self, otherrev, repo, parent):
+    def __init__(self, repoagent, otherrev, parent=None):
         super(MergeDialog, self).__init__(parent)
+        self._repoagent = repoagent
         f = self.windowFlags()
         self.setWindowFlags(f & ~Qt.WindowContextHelpButtonHint)
-
-        self.otherrev = str(otherrev)
-        self.localrev = str(repo['.'].rev())
-
-        self.setWindowTitle(_('Merge - %s') % repo.displayname)
+        self.setWindowTitle(_('Merge - %s') % repoagent.displayName())
         self.setWindowIcon(qtlib.geticon('hg-merge'))
         self.setOption(QWizard.NoBackButtonOnStartPage, True)
         self.setOption(QWizard.NoBackButtonOnLastPage, True)
         self.setOption(QWizard.IndependentPages, True)
 
         # set pages
-        summarypage = SummaryPage(repo, self)
+        summarypage = SummaryPage(repoagent, str(otherrev), self)
         self.addPage(summarypage)
-        self.addPage(MergePage(repo, self))
-        self.addPage(CommitPage(repo, self))
-        self.addPage(ResultPage(repo, self))
+        self.addPage(MergePage(repoagent, str(otherrev), self))
+        self.addPage(CommitPage(repoagent, self))
+        self.addPage(ResultPage(repoagent, self))
         self.currentIdChanged.connect(self.pageChanged)
 
         # move focus to "Next" button so that "Cancel" doesn't eat Enter key
@@ -48,15 +43,38 @@ class MergeDialog(QWizard):
 
         self.resize(QSize(700, 489).expandedTo(self.minimumSizeHint()))
 
-        repo.repositoryChanged.connect(self.repositoryChanged)
-        repo.configChanged.connect(self.configChanged)
-        self.repo = repo
+        repoagent.repositoryChanged.connect(self.repositoryChanged)
+        repoagent.configChanged.connect(self.configChanged)
 
+        self._readSettings()
+
+    def _readSettings(self):
+        qs = QSettings()
+        qs.beginGroup('merge')
+        for n in ['autoadvance', 'skiplast']:
+            self.setField(n, qs.value(n, False))
+        repo = self._repoagent.rawRepo()
+        n = 'autoresolve'
+        self.setField(n, repo.ui.configbool('tortoisehg', n,
+                                            qs.value(n, True).toBool()))
+        qs.endGroup()
+
+    def _writeSettings(self):
+        qs = QSettings()
+        qs.beginGroup('merge')
+        for n in ['autoadvance', 'autoresolve', 'skiplast']:
+            qs.setValue(n, self.field(n))
+        qs.endGroup()
+
+    @pyqtSlot()
     def repositoryChanged(self):
-        self.currentPage().repositoryChanged()
+        if self.currentPage():
+            self.currentPage().repositoryChanged()
 
+    @pyqtSlot()
     def configChanged(self):
-        self.currentPage().configChanged()
+        if self.currentPage():
+            self.currentPage().configChanged()
 
     def pageChanged(self, id):
         if id != -1:
@@ -66,16 +84,19 @@ class MergeDialog(QWizard):
         if self.currentPage().canExit():
             super(MergeDialog, self).reject()
 
-    def done(self, ret):
-        self.repo.repositoryChanged.disconnect(self.repositoryChanged)
-        self.repo.configChanged.disconnect(self.configChanged)
-        super(MergeDialog, self).done(ret)
+    def done(self, r):
+        self._writeSettings()
+        super(MergeDialog, self).done(r)
 
 
 class BasePage(QWizardPage):
-    def __init__(self, repo, parent):
+    def __init__(self, repoagent, parent):
         super(BasePage, self).__init__(parent)
-        self.repo = repo
+        self._repoagent = repoagent
+
+    @property
+    def repo(self):
+        return self._repoagent.rawRepo()
 
     def validatePage(self):
         'user pressed NEXT button, can we proceed?'
@@ -113,16 +134,12 @@ class BasePage(QWizardPage):
 class SummaryPage(BasePage):
     refreshFinished = pyqtSignal()
 
-    def __init__(self, repo, parent):
-        super(SummaryPage, self).__init__(repo, parent)
-        self.clean = None
-        self.th = None
+    def __init__(self, repoagent, otherrev, parent):
+        super(SummaryPage, self).__init__(repoagent, parent)
+        self._wctxcleaner = wctxcleaner.WctxCleaner(repoagent, self)
+        self._wctxcleaner.checkStarted.connect(self._onCheckStarted)
+        self._wctxcleaner.checkFinished.connect(self._onCheckFinished)
 
-    ### Override Methods ###
-
-    def initializePage(self):
-        if self.layout():
-            return
         self.setTitle(_('Prepare to merge'))
         self.setSubTitle(_('Verify merge targets and ensure your working '
                            'directory is clean.'))
@@ -142,19 +159,14 @@ class SummaryPage(BasePage):
         ## merge target
         other_sep = qtlib.LabeledSeparator(_('Merge from (other revision)'))
         self.layout().addWidget(other_sep)
-        try:
-            otherCsInfo = create(self.wizard().otherrev)
-            self.layout().addWidget(otherCsInfo)
-            self.otherCsInfo = otherCsInfo
-        except error.RepoLookupError:
-            qtlib.InfoMsgBox(_('Unable to merge'),
-                             _('Merge revision not specified or not found'))
-            QTimer.singleShot(0, self.wizard().close)
+        otherCsInfo = create(otherrev)
+        self.layout().addWidget(otherCsInfo)
+        self.otherCsInfo = otherCsInfo
 
         ## current revision
         local_sep = qtlib.LabeledSeparator(_('Merge to (working directory)'))
         self.layout().addWidget(local_sep)
-        localCsInfo = create(self.wizard().localrev)
+        localCsInfo = create(str(repo['.'].rev()))
         self.layout().addWidget(localCsInfo)
         self.localCsInfo = localCsInfo
 
@@ -189,7 +201,7 @@ class SummaryPage(BasePage):
                  'or <a href="discard"><b>discard</b></a> changes.')
         wd_text = QLabel(text)
         wd_text.setWordWrap(True)
-        wd_text.linkActivated.connect(self.onLinkActivated)
+        wd_text.linkActivated.connect(self._wctxcleaner.runCleaner)
         self.wd_text = wd_text
         self.groups.add(wd_text, 'dirty')
         self.layout().addWidget(wd_text)
@@ -206,35 +218,24 @@ class SummaryPage(BasePage):
         self.groups.add(force_chk, 'dirty')
         wdbox.addWidget(force_chk)
 
-        ### options
-        expander = qtlib.ExpanderLabel(_('Options'), False)
-        expander.expanded.connect(self.toggleShowOptions)
-        self.layout().addWidget(expander)
-        self.expander = expander
-
         ### discard option
         discard_chk = QCheckBox(_('Discard all changes from merge target '
                                   '(other) revision'))
         self.registerField('discard', discard_chk)
         self.layout().addWidget(discard_chk)
-        self.discard_chk = discard_chk
 
         ## auto-resolve
         autoresolve_chk = QCheckBox(_('Automatically resolve merge conflicts '
                                       'where possible'))
-        autoresolve_chk.setChecked(
-            repo.ui.configbool('tortoisehg', 'autoresolve', False))
         self.registerField('autoresolve', autoresolve_chk)
         self.layout().addWidget(autoresolve_chk)
-        self.autoresolve_chk = autoresolve_chk
 
         self.groups.set_visible(False, 'dirty')
         self.groups.set_visible(False, 'merged')
-        self.toggleShowOptions(self.expander.is_expanded())
 
     def isComplete(self):
         'should Next button be sensitive?'
-        return self.clean or self.field('force').toBool()
+        return self._wctxcleaner.isClean() or self.field('force').toBool()
 
     def validatePage(self):
         'validate that we can continue with the merge'
@@ -248,25 +249,19 @@ class SummaryPage(BasePage):
                       % (self.otherCsInfo.get_data('revid')),
                          labels=labels, parent=self):
                 return False
-        return super(SummaryPage, self).validatePage();
+        return super(SummaryPage, self).validatePage()
 
     ## custom methods ##
 
-    def toggleShowOptions(self, visible):
-        self.discard_chk.setShown(visible)
-        self.autoresolve_chk.setShown(visible)
-
     def repositoryChanged(self):
         'repository has detected a change to changelog or parents'
         pctx = self.repo['.']
         self.localCsInfo.update(pctx)
-        self.wizard().localrev = str(pctx.rev())
 
     def canExit(self):
         'can merge tool be closed?'
-        if self.th is not None and self.th.isRunning():
-            self.th.cancel()
-            self.th.wait()
+        if self._wctxcleaner.isChecking():
+            self._wctxcleaner.cancelCheck()
         return True
 
     def currentPage(self):
@@ -274,21 +269,18 @@ class SummaryPage(BasePage):
         self.refresh()
 
     def refresh(self):
-        if self.th is None:
-            self.th = CheckThread(self.repo, self)
-            self.th.finished.connect(self.threadFinished)
-        if self.th.isRunning():
-            return
-        self.groups.set_visible(True, 'prog')
-        self.th.start()
+        self._wctxcleaner.check()
 
-    def threadFinished(self):
+    @pyqtSlot()
+    def _onCheckStarted(self):
+        self.groups.set_visible(True, 'prog')
+
+    @pyqtSlot(bool, int)
+    def _onCheckFinished(self, clean, parents):
         self.groups.set_visible(False, 'prog')
-        if self.th.canceled:
+        if self._wctxcleaner.isCheckCanceled():
             return
-        dirty, parents = self.th.results
-        self.clean = not dirty
-        if dirty:
+        if not clean:
             self.groups.set_visible(parents == 2, 'merged')
             self.groups.set_visible(parents == 1, 'dirty')
             self.wd_status.set_status(_('<b>Uncommitted local changes '
@@ -300,158 +292,134 @@ class SummaryPage(BasePage):
         self.completeChanged.emit()
         self.refreshFinished.emit()
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def onLinkActivated(self, cmd):
-        cmd = hglib.fromunicode(cmd)
-        repo = self.repo
-        if cmd == 'commit':
-            from tortoisehg.hgqt import commit
-            dlg = commit.CommitDialog(repo, [], {}, self)
-            dlg.finished.connect(dlg.deleteLater)
-            dlg.exec_()
-            self.refresh()
-        elif cmd == 'shelve':
-            from tortoisehg.hgqt import shelve
-            dlg = shelve.ShelveDialog(repo, self.wizard())
-            dlg.finished.connect(dlg.deleteLater)
-            dlg.exec_()
-            self.refresh()
-        elif cmd.startswith('discard'):
-            if cmd != 'discard:noconfirm':
-                labels = [(QMessageBox.Yes, _('&Discard')),
-                          (QMessageBox.No, _('Cancel'))]
-                if not qtlib.QuestionMsgBox(_('Confirm Discard'),
-                         _('Discard outstanding changes to working directory?'),
-                         labels=labels, parent=self):
-                    return
-            def finished(ret):
-                repo.decrementBusyCount()
-                self.refresh()
-            cmdline = ['update', '--clean', '--repository', repo.root,
-                       '--rev', '.']
-            self.runner = cmdui.Runner(True, self)
-            self.runner.commandFinished.connect(finished)
-            repo.incrementBusyCount()
-            self.runner.run(cmdline)
-        elif cmd == 'view':
-            dlg = status.StatusDialog(repo, [], {}, self)
-            dlg.exec_()
-            self.refresh()
-        elif cmd == 'skip':
+        if cmd == 'skip':
             self.wizard().next()
         else:
-            raise 'unknown command: %s' % cmd
+            self._wctxcleaner.runCleaner(cmd)
 
 
 class MergePage(BasePage):
-    def __init__(self, repo, parent):
-        super(MergePage, self).__init__(repo, parent)
+    def __init__(self, repoagent, otherrev, parent):
+        super(MergePage, self).__init__(repoagent, parent)
+        self._otherrev = otherrev
         self.mergecomplete = False
 
         self.setTitle(_('Merging...'))
         self.setSubTitle(_('All conflicting files will be marked unresolved.'))
         self.setLayout(QVBoxLayout())
 
-        self.cmd = cmdui.Widget(True, False, self)
-        self.cmd.commandFinished.connect(self.onCommandFinished)
-        self.cmd.setShowOutput(True)
-        self.layout().addWidget(self.cmd)
+        self._cmdsession = cmdcore.nullCmdSession()
+        self._cmdlog = cmdui.LogWidget(self)
+        self.layout().addWidget(self._cmdlog)
 
         self.reslabel = QLabel()
         self.reslabel.linkActivated.connect(self.onLinkActivated)
         self.reslabel.setWordWrap(True)
         self.layout().addWidget(self.reslabel)
 
-        self.autonext = QCheckBox(_('Automatically advance to next page '
-                                    'when merge is complete.'))
-        checked = QSettings().value('merge/autoadvance', False).toBool()
-        self.autonext.setChecked(checked)
-        self.autonext.toggled.connect(self.tryAutoAdvance)
-        self.layout().addWidget(self.autonext)
+        autonext = QCheckBox(_('Automatically advance to next page '
+                               'when merge is complete.'))
+        autonext.clicked.connect(self.tryAutoAdvance)
+        self.registerField('autoadvance', autonext)
+        self.layout().addWidget(autonext)
 
     def currentPage(self):
         super(MergePage, self).currentPage()
-        if self.field('discard').toBool():
-            # '.' is safer than self.localrev, in case the user has
-            # pulled a fast one on us and updated from the CLI
-            cmdline = ['--repository', self.repo.root, 'debugsetparents',
-                       '.', self.wizard().otherrev]
-        else:
-            cmdline = ['--repository', self.repo.root, 'merge', '--verbose']
-            if self.field('force').toBool():
-                cmdline.append('--force')
-            tool = self.field('autoresolve').toBool() and 'merge' or 'fail'
-            cmdline += ['--tool=internal:' + tool]
-            cmdline.append(self.wizard().otherrev)
-
-        if len(self.repo.parents()) == 1:
-            self.repo.incrementBusyCount()
-            self.cmd.core.clearOutput()
-            self.cmd.run(cmdline)
-        else:
+        if len(self.repo.parents()) > 1:
             self.mergecomplete = True
             self.completeChanged.emit()
+            return
+
+        discard = self.field('discard').toBool()
+        rev = hglib.tounicode(self._otherrev)
+        cfgs = []
+        if discard:
+            tool = ':local'
+            # disable changed/deleted prompt because we'll revert changes
+            cfgs.append('ui.interactive=False')
+        else:
+            tool = self.field('autoresolve').toBool() and ':merge' or ':fail'
+        cmdlines = [hglib.buildcmdargs('merge', rev, verbose=True, tool=tool,
+                                       force=self.field('force').toBool(),
+                                       config=cfgs)]
+        if discard:
+            # revert files added/removed at other side
+            cmdlines.append(hglib.buildcmdargs('revert', rev='.', all=True))
+
+        self._cmdlog.clearLog()
+        self._cmdsession = sess = self._repoagent.runCommandSequence(cmdlines,
+                                                                     self)
+        sess.commandFinished.connect(self.onCommandFinished)
+        sess.outputReceived.connect(self._cmdlog.appendLog)
 
     def isComplete(self):
         'should Next button be sensitive?'
         if not self.mergecomplete:
             return False
-        count = 0
+        ucount = 0
+        rcount = 0
         for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
             if status == 'u':
-                count += 1
-        if count:
+                ucount += 1
+            if status == 'r':
+                rcount += 1
+        if ucount:
             if self.field('autoresolve').toBool():
                 # if autoresolve is enabled, we know these were real conflicts
                 self.reslabel.setText(_('%d files have <b>merge conflicts</b> '
                                         'that must be <a href="resolve">'
-                                        '<b>resolved</b></a>') % count)
+                                        '<b>resolved</b></a>') % ucount)
             else:
                 # else give a calmer indication of conflicts
                 self.reslabel.setText(_('%d files were modified on both '
                                         'branches and must be <a href="resolve">'
-                                        '<b>resolved</b></a>') % count)
+                                        '<b>resolved</b></a>') % ucount)
             return False
+        elif rcount:
+            self.reslabel.setText(_('No merge conflicts, ready to commit or '
+                                    '<a href="resolve"><b>review</b></a>'))
         else:
             self.reslabel.setText(_('No merge conflicts, ready to commit'))
         return True
 
+    @pyqtSlot(bool)
     def tryAutoAdvance(self, checked):
         if checked and self.isComplete():
             self.wizard().next()
 
-    def cleanupPage(self):
-        QSettings().setValue('merge/autoadvance', self.autonext.isChecked())
-
+    @pyqtSlot(int)
     def onCommandFinished(self, ret):
-        self.repo.decrementBusyCount()
+        sess = self._cmdsession
         if ret in (0, 1):
             self.mergecomplete = True
-            if self.autonext.isChecked():
+            if self.field('autoadvance').toBool() and not sess.warningString():
                 self.tryAutoAdvance(True)
             self.completeChanged.emit()
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def onLinkActivated(self, cmd):
         if cmd == 'resolve':
-            dlg = resolve.ResolveDialog(self.repo, self)
-            dlg.finished.connect(dlg.deleteLater)
+            dlg = resolve.ResolveDialog(self._repoagent, self)
             dlg.exec_()
-            if self.autonext.isChecked():
+            if self.field('autoadvance').toBool():
                 self.tryAutoAdvance(True)
             self.completeChanged.emit()
 
 
 class CommitPage(BasePage):
 
-    def __init__(self, repo, parent):
-        super(CommitPage, self).__init__(repo, parent)
+    def __init__(self, repoagent, parent):
+        super(CommitPage, self).__init__(repoagent, parent)
 
         self.setTitle(_('Commit merge results'))
         self.setSubTitle(' ')
         self.setLayout(QVBoxLayout())
         self.setCommitPage(True)
 
+        repo = repoagent.rawRepo()
+
         # csinfo
         def label_func(widget, item, ctx):
             if item == 'rev':
@@ -518,10 +486,10 @@ class CommitPage(BasePage):
         self.layout().addWidget(msgEntry)
         self.msgEntry = msgEntry
 
-        self.cmd = cmdui.Widget(True, False, self)
-        self.cmd.commandFinished.connect(self.onCommandFinished)
-        self.cmd.setShowOutput(False)
-        self.layout().addWidget(self.cmd)
+        self._cmdsession = cmdcore.nullCmdSession()
+        self._cmdlog = cmdui.LogWidget(self)
+        self._cmdlog.hide()
+        self.layout().addWidget(self._cmdlog)
 
         self.delayednext = False
 
@@ -533,14 +501,13 @@ class CommitPage(BasePage):
         actionEnter.triggered.connect(tryperform)
         self.addAction(actionEnter)
 
-        self.skiplast = QCheckBox(_('Skip final confirmation page, '
-                                    'close after commit.'))
-        checked = QSettings().value('merge/skiplast', False).toBool()
-        self.skiplast.setChecked(checked)
-        self.layout().addWidget(self.skiplast)
+        skiplast = QCheckBox(_('Skip final confirmation page, '
+                               'close after commit.'))
+        self.registerField('skiplast', skiplast)
+        self.layout().addWidget(skiplast)
 
         hblayout = QHBoxLayout()
-        self.opts = commit.readrepoopts(self.repo)
+        self.opts = commit.readopts(self.repo.ui)
         self.optionsbtn = QPushButton(_('Commit Options'))
         self.optionsbtn.clicked.connect(self.details)
         hblayout.addWidget(self.optionsbtn)
@@ -565,7 +532,6 @@ class CommitPage(BasePage):
 
     def cleanupPage(self):
         s = QSettings()
-        s.setValue('merge/skiplast', self.skiplast.isChecked())
         self.msgEntry.saveSettings(s, 'merge/message')
 
     def currentPage(self):
@@ -573,23 +539,13 @@ class CommitPage(BasePage):
         self.wizard().setOption(QWizard.NoDefaultButton, True)
         self.mergeCsInfo.update()  # show post-merge state
 
-        engmsg = self.repo.ui.configbool('tortoisehg', 'engmsg', False)
-        wctx = self.repo[None]
-        if wctx.p1().branch() == wctx.p2().branch():
-            msgset = i18n.keepgettext()._('Merge')
-            text = engmsg and msgset['id'] or msgset['str']
-            text = unicode(text)
-        else:
-            msgset = i18n.keepgettext()._('Merge with %s')
-            text = engmsg and msgset['id'] or msgset['str']
-            text = unicode(text) % hglib.tounicode(wctx.p2().branch())
-        self.msgEntry.setText(text)
+        self.msgEntry.setText(commit.mergecommitmessage(self.repo))
         self.msgEntry.moveCursorToEnd()
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def onLinkActivated(self, cmd):
         if cmd == 'view':
-            dlg = status.StatusDialog(self.repo, [], {}, self)
+            dlg = status.StatusDialog(self._repoagent, [], {}, self)
             dlg.exec_()
             self.refresh()
 
@@ -597,69 +553,52 @@ class CommitPage(BasePage):
         return len(self.repo.parents()) == 2 and len(self.msgEntry.text()) > 0
 
     def validatePage(self):
-
-        if self.cmd.core.running():
+        if not self._cmdsession.isFinished():
             return False
 
         if len(self.repo.parents()) == 1:
             # commit succeeded, repositoryChanged() called wizard().next()
-            if self.skiplast.isChecked():
+            if self.field('skiplast').toBool():
                 self.wizard().close()
             return True
 
-        user = qtlib.getCurrentUsername(self, self.repo, self.opts)
+        user = hglib.tounicode(qtlib.getCurrentUsername(self, self.repo,
+                                                        self.opts))
         if not user:
             return False
 
         self.setTitle(_('Committing...'))
         self.setSubTitle(_('Please wait while committing merged files.'))
 
-        message = hglib.fromunicode(self.msgEntry.text())
-        cmdline = ['commit', '--verbose', '--message', message,
-                   '--repository', self.repo.root, '--user', user]
-        if self.opts.get('recurseinsubrepos'):
-            cmdline.append('--subrepos')
-        try:
-            date = self.opts.get('date')
-            if date:
-                util.parsedate(date)
-                dcmd = ['--date', date]
-            else:
-                dcmd = []
-        except error.Abort, e:
-            if e.hint:
-                err = _('%s (hint: %s)') % (hglib.tounicode(str(e)),
-                                            hglib.tounicode(e.hint))
-            else:
-                err = hglib.tounicode(str(e))
-            qtlib.WarningMsgBox(_('TortoiseHg Merge Commit'),
-                _('Error creating interpreting commit date (%s).\n'
-                  'Using current date instead.'), err)
-            dcmd = []
-
-        cmdline += dcmd
-
-        commandlines = [cmdline]
+        opts = {'verbose': True,
+                'message': self.msgEntry.text(),
+                'user': user,
+                'subrepos': bool(self.opts.get('recurseinsubrepos')),
+                'date': hglib.tounicode(self.opts.get('date')),
+                }
+        commandlines = [hglib.buildcmdargs('commit', **opts)]
         pushafter = self.repo.ui.config('tortoisehg', 'cipushafter')
         if pushafter:
-            cmd = ['push', '--repository', self.repo.root, pushafter]
+            cmd = ['push', hglib.tounicode(pushafter)]
             commandlines.append(cmd)
-        self.repo.incrementBusyCount()
-        self.cmd.setShowOutput(True)
-        self.cmd.run(*commandlines)
+        self._cmdlog.show()
+        sess = self._repoagent.runCommandSequence(commandlines, self)
+        self._cmdsession = sess
+        sess.commandFinished.connect(self.onCommandFinished)
+        sess.outputReceived.connect(self._cmdlog.appendLog)
         return False
 
     def repositoryChanged(self):
         'repository has detected a change to changelog or parents'
         if len(self.repo.parents()) == 1:
-            if self.cmd.core.running():
+            if not self._cmdsession.isFinished():
                 # call self.wizard().next() after the current command finishes
                 self.delayednext = True
             else:
                 self.wizard().next()
 
-    def onCommandFinished(self, ret):
-        self.repo.decrementBusyCount()
+    @pyqtSlot()
+    def onCommandFinished(self):
         if self.delayednext:
             self.delayednext = False
             self.wizard().next()
@@ -674,7 +613,8 @@ class CommitPage(BasePage):
 
     def details(self):
         self.userhist = self.readUserHistory()
-        dlg = commit.DetailsDialog(self.opts, self.userhist, self, mode='merge')
+        dlg = commit.DetailsDialog(self._repoagent, self.opts, self.userhist,
+                                   self, mode='merge')
         dlg.finished.connect(dlg.deleteLater)
         dlg.setWindowFlags(Qt.Sheet)
         dlg.setWindowModality(Qt.WindowModal)
@@ -683,8 +623,8 @@ class CommitPage(BasePage):
             self.refresh()
 
 class ResultPage(BasePage):
-    def __init__(self, repo, parent):
-        super(ResultPage, self).__init__(repo, parent)
+    def __init__(self, repoagent, parent):
+        super(ResultPage, self).__init__(repoagent, parent)
         self.setTitle(_('Finished'))
         self.setSubTitle(' ')
         self.setFinalPage(True)
@@ -701,43 +641,3 @@ class ResultPage(BasePage):
         super(ResultPage, self).currentPage()
         self.mergeCsInfo.update(self.repo['tip'])
         self.wizard().setOption(QWizard.NoCancelButton, True)
-
-
-class CheckThread(QThread):
-    def __init__(self, repo, parent):
-        QThread.__init__(self, parent)
-        self.repo = hg.repository(repo.ui, repo.root)
-        self.results = (False, 1)
-        self.canceled = False
-
-    def run(self):
-        self.repo.dirstate.invalidate()
-        unresolved = False
-        for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
-            if self.canceled:
-                return
-            if status == 'u':
-                unresolved = True
-                break
-        wctx = self.repo[None]
-        try:
-            dirty = bool(wctx.dirty()) or unresolved
-            self.results = (dirty, len(wctx.parents()))
-        except EnvironmentError:
-            self.results = (True, len(wctx.parents()))
-
-    def cancel(self):
-        self.canceled = True
-
-def run(ui, *pats, **opts):
-    from tortoisehg.util import paths
-    rev = opts.get('rev') or None
-    if not rev and len(pats):
-        rev = pats[0]
-    if not rev:
-        import sys
-        qtlib.InfoMsgBox(_('Unable to merge'),
-                         _('Merge revision not specified or not found'))
-        sys.exit()
-    repo = thgrepo.repository(ui, path=paths.find_root())
-    return MergeDialog(rev, repo, None)
