@@ -10,8 +10,8 @@ import os
 from mercurial import commands, hg, ui, util
 
 from tortoisehg.util import hglib, paths
-from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib, repotreemodel, clone, settings
+from tortoisehg.util.i18n import _
+from tortoisehg.hgqt import qtlib, repotreemodel, settings
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
@@ -24,14 +24,16 @@ def settingsfilename():
 
 
 class RepoTreeView(QTreeView):
-    showMessage = pyqtSignal(QString)
+    showMessage = pyqtSignal(str)
     menuRequested = pyqtSignal(object, object)
-    openRepo = pyqtSignal(QString, bool)
+    openRepo = pyqtSignal(str, bool)
     dropAccepted = pyqtSignal()
     updateSettingsFile = pyqtSignal()
 
     def __init__(self, parent):
         QTreeView.__init__(self, parent, allColumnsShowFocus=True)
+        if qtlib.IS_RETINA:
+            self.setIconSize(qtlib.treeviewRetinaIconSize())
         self.selitem = None
         self.msg = ''
 
@@ -167,15 +169,7 @@ class RepoTreeView(QTreeView):
 
     def mouseDoubleClickEvent(self, event):
         if self.selitem and self.selitem.internalPointer().isRepo():
-            # We can only open mercurial repositories and subrepositories
-            repotype = self.selitem.internalPointer().repotype()
-            if repotype == 'hg':
-                self.showFirstTabOrOpen()
-            else:
-                qtlib.WarningMsgBox(
-                    _('Unsupported repository type (%s)') % repotype,
-                    _('Cannot open non Mercurial repositories or subrepositories'),
-                    parent=self)
+            self.showFirstTabOrOpen()
         else:
             # a double-click on non-repo rows opens an editor
             super(RepoTreeView, self).mouseDoubleClickEvent(event)
@@ -195,8 +189,17 @@ class RepoTreeView(QTreeView):
     def showFirstTabOrOpen(self):
         'Enter or double click events, show existing or open a new repowidget'
         if self.selitem and self.selitem.internalPointer().isRepo():
-            root = self.selitem.internalPointer().rootpath()
-            self.openRepo.emit(hglib.tounicode(root), True)
+            # We can only open mercurial repositories and subrepositories
+            repotype = self.selitem.internalPointer().repotype()
+            if repotype == 'hg':
+                root = self.selitem.internalPointer().rootpath()
+                self.openRepo.emit(hglib.tounicode(root), True)
+            else:
+                qtlib.WarningMsgBox(
+                    _('Unsupported repository type (%s)') % repotype,
+                    _('Cannot open non Mercurial repositories or '
+                      'subrepositories'),
+                    parent=self)
 
     def removeSelected(self):
         'remove selected repository'
@@ -222,15 +225,17 @@ class RepoTreeView(QTreeView):
 
 class RepoRegistryView(QDockWidget):
 
-    showMessage = pyqtSignal(QString)
-    openRepo = pyqtSignal(QString, bool)
-    removeRepo = pyqtSignal(QString)
-    progressReceived = pyqtSignal(QString, object, QString, QString, object)
+    showMessage = pyqtSignal(str)
+    openRepo = pyqtSignal(str, bool)
+    removeRepo = pyqtSignal(str)
+    cloneRepoRequested = pyqtSignal(str)
+    progressReceived = pyqtSignal(str, object, str, str, object)
 
     def __init__(self, repomanager, parent):
         QDockWidget.__init__(self, parent)
 
         self._repomanager = repomanager
+        repomanager.repositoryOpened.connect(self._addAndScanRepo)
         self.watcher = None
         self._setupSettingActions()
 
@@ -285,7 +290,7 @@ class RepoRegistryView(QDockWidget):
 
     @pyqtSlot()
     def _initView(self):
-        self.expand()
+        self._loadExpandedState()
         self._updateColumnVisibility()
         if self._isSettingEnabled('showSubrepos'):
             self._scanAllRepos()
@@ -305,6 +310,15 @@ class RepoRegistryView(QDockWidget):
         for key, action in self._settingactions.iteritems():
             s.setValue(key, action.isChecked())
         s.endGroup()
+        s.beginGroup('reporegistry')
+        self._writeExpandedState(s)
+        s.endGroup()
+
+    def _loadExpandedState(self):
+        s = QSettings()
+        s.beginGroup('reporegistry')
+        self._readExpandedState(s)
+        s.endGroup()
 
     def _setupSettingActions(self):
         settingtable = [
@@ -374,16 +388,27 @@ class RepoRegistryView(QDockWidget):
         oldmodel.deleteLater()
         if self._isSettingEnabled('showSubrepos'):
             self._scanAllRepos()
-        self.expand()
+        self._loadExpandedState()
         if activeroot:
             self.setActiveTabRepo(activeroot)
         self._reloadModelTimer.stop()
 
-    def expand(self):
-        self.tview.expandToDepth(0)
+    def _readExpandedState(self, s):
+        model = self.tview.model()
+        for path in s.value('expanded').toStringList():
+            self.tview.expand(model.indexFromItemPath(path))
 
-    def addRepo(self, uroot):
+    def _writeExpandedState(self, s):
+        model = self.tview.model()
+        paths = [model.itemPath(i) for i in model.persistentIndexList()
+                 if i.column() == 0 and self.tview.isExpanded(i)]
+        s.setValue('expanded', paths)
+
+    # TODO: better to handle repositoryOpened signal by model
+    @pyqtSlot(str)
+    def _addAndScanRepo(self, uroot):
         """Add repo if not exists; called when the workbench has opened it"""
+        uroot = unicode(uroot)
         m = self.tview.model()
         knownindex = m.indexFromRepoRoot(uroot)
         if knownindex.isValid():
@@ -393,6 +418,14 @@ class RepoRegistryView(QDockWidget):
             self._scanAddedRepo(index)
             self.updateSettingsFile()
 
+    def addClonedRepo(self, root, sourceroot):
+        """Add repo to the same group as the source"""
+        m = self.tview.model()
+        src = m.indexFromRepoRoot(sourceroot, standalone=True)
+        if src.isValid() and not m.isKnownRepoRoot(root):
+            index = m.addRepo(root, parent=src.parent())
+            self._scanAddedRepo(index)
+
     def setActiveTabRepo(self, root):
         """"The selected tab has changed on the workbench"""
         m = self.tview.model()
@@ -427,9 +460,9 @@ class RepoRegistryView(QDockWidget):
                 _("Create a new group"), self.newGroup),
              ("rename", _("Re&name"), None,
                 _("Rename the entry"), self.startRename),
-             ("settings", _("Settin&gs"), 'settings_user',
+             ("settings", _("Settin&gs"), 'thg-userconfig',
                 _("View the repository's settings"), self.startSettings),
-             ("remove", _("Re&move from Registry"), 'menudelete',
+             ("remove", _("Re&move from Registry"), 'hg-strip',
                 _("Remove the node and all its subnodes."
                   " Repositories are not deleted from disk."),
                   self.removeSelected),
@@ -502,10 +535,7 @@ class RepoRegistryView(QDockWidget):
 
     def cloneRepo(self):
         root = self.selitem.internalPointer().rootpath()
-        d = clone.CloneDialog(args=[root, root + '-clone'], parent=self)
-        d.finished.connect(d.deleteLater)
-        d.clonedRepository.connect(self._openClone)
-        d.show()
+        self.cloneRepoRequested.emit(hglib.tounicode(root))
 
     def explore(self):
         root = self.selitem.internalPointer().rootpath()
@@ -656,9 +686,9 @@ class RepoRegistryView(QDockWidget):
 
     def removeSubrepo(self):
         'menu action handler for removing an existing subrepository'
-        path = hglib.tounicode(self.selitem.internalPointer().rootpath())
-        containerpath = os.path.normpath(os.path.join(path, '..'))
-        root = paths.find_root(containerpath)
+        model = self.tview.model()
+        path = model.repoRoot(self.selitem)
+        root = model.repoRoot(self.selitem.parent())
         relsubpath = os.path.normcase(os.path.normpath(path[1+len(root):]))
         hgsubfilename = os.path.join(root, '.hgsub')
 
@@ -723,15 +753,6 @@ class RepoRegistryView(QDockWidget):
         for root in self.selitem.internalPointer().childRoots():
             self.openRepo.emit(hglib.tounicode(root), False)
 
-    @pyqtSlot(unicode, unicode)
-    def _openClone(self, root, sourceroot):
-        m = self.tview.model()
-        src = m.indexFromRepoRoot(sourceroot, standalone=True)
-        if src.isValid() and not m.isKnownRepoRoot(root):
-            index = m.addRepo(root, parent=src.parent())
-            self._scanAddedRepo(index)
-        self.open(root)
-
     def open(self, root=None):
         'open context menu action, open repowidget unconditionally'
         if not root:
@@ -822,7 +843,7 @@ class RepoRegistryView(QDockWidget):
                   '<br><br><i>%s</i>')
                 % (root, "<br>".join(invalidpaths)), parent=self)
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def scanRepo(self, uroot):
         m = self.tview.model()
         index = m.indexFromRepoRoot(uroot)
@@ -834,7 +855,7 @@ class RepoRegistryView(QDockWidget):
         indexes = m.indexesOfRepoItems(standalone=True)
         if not self._isSettingEnabled('showNetworkSubrepos'):
             indexes = [idx for idx in indexes
-                       if not paths.netdrive_status(m.repoRoot(idx))]
+                       if paths.is_on_fixed_drive(m.repoRoot(idx))]
 
         topic = _('Updating repository registry')
         for n, idx in enumerate(indexes):
