@@ -11,7 +11,7 @@ from mercurial import ui, util, error, extensions, scmutil, phases
 
 from tortoisehg.util import hglib, paths, wconfig, i18n, editor
 from tortoisehg.util import terminal, gpg
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 from tortoisehg.hgqt import qtlib, qscilib, thgrepo, customtools, fileencoding
 
 from PyQt4.QtCore import *
@@ -402,7 +402,9 @@ class PathBrowser(QWidget):
         self.opts = opts
 
         self.lineEdit = QLineEdit()
-        completer = QCompleter(self)
+        # use QCompleter(model, parent) to avoid ownership bug of
+        # QCompleter(parent /TransferBack/) in PyQt<4.11.4
+        completer = QCompleter(None, self)
         completer.setModel(QDirModel(completer))
         self.lineEdit.setCompleter(completer)
 
@@ -552,7 +554,7 @@ class _fi(object):
             return self.visible()
 
 INFO = (
-({'name': 'general', 'label': 'TortoiseHg', 'icon': 'thg_logo'}, (
+({'name': 'general', 'label': 'TortoiseHg', 'icon': 'thg'}, (
     _fi(_('UI Language'), 'tortoisehg.ui.language',
         (genDeferredCombo, i18n.availablelanguages),
         _('Specify your preferred user interface language (restart needed)'),
@@ -582,7 +584,8 @@ INFO = (
         _('Specify the command to launch your preferred terminal shell '
           'application. If the value includes the string %(reponame)s, the '
           'name of the repository will be substituted in place of '
-          '%(reponame)s. (restart needed)<br>'
+          '%(reponame)s. Similarly, %(root)s will be the full path to the '
+          'repository. (restart needed)<br>'
           'Default, Windows: cmd.exe /K title %(reponame)s<br>'
           'Default, OS X: not set<br>'
           'Default, other: xterm -T "%(reponame)s"'),
@@ -622,9 +625,13 @@ INFO = (
           'even if the edits are to different parts of the file. In either '
           'case, when conflicts occur, the user will be invited to review and '
           'resolve changes manually. Default: True.')),
+    _fi(_('New Repo Skeleton'), 'tortoisehg.initskel', genPathBrowser,
+        _('If specified, files in the directory, e.g. .hgignore, are copied '
+          'to the newly-created repository.'),
+        globalonly=True),
     )),
 
-({'name': 'log', 'label': _('Workbench'), 'icon': 'menulog'}, (
+({'name': 'log', 'label': _('Workbench'), 'icon': 'hg-log'}, (
     _fi(_('Single Workbench Window'), 'tortoisehg.workbench.single',
         genBoolRBGroup,
         _('Select whether you want to have a single workbench window. '
@@ -651,9 +658,7 @@ INFO = (
           'Default: True'),
         globalonly=True),
     _fi(_('Author Coloring'), 'tortoisehg.authorcolor', genBoolRBGroup,
-        _('Color changesets by author name.  If not enabled, '
-          'the changes are colored green for merge, red for '
-          'non-trivial parents, black for normal. '
+        _('Color changesets by author name. '
           'Default: False')),
     _fi(_('Full Authorname'), 'tortoisehg.fullauthorname', genBoolRBGroup,
         _('Show full authorname in Logview. If not enabled, '
@@ -674,7 +679,7 @@ INFO = (
         restartneeded=True, globalonly=True),
     _fi(_('Long Summary'), 'tortoisehg.longsummary', genBoolRBGroup,
         _('If true, concatenate multiple lines of changeset summary '
-          'until they reach 80 characters. '
+          'and truncate them at 80 characters as necessary. '
           'Default: False')),
     _fi(_('Log Batch Size'), 'tortoisehg.graphlimit', genIntEditCombo,
         _('The number of revisions to read and display in the '
@@ -692,7 +697,7 @@ INFO = (
           '@ character, and \\n to a linefeed. '
           'Default: None (leave blank)')),
     _fi(_('Hide Tags'), 'tortoisehg.hidetags', genEditCombo,
-        _('Space separated list of tags that will not be shown.'
+        _('Space separated list of tags that will not be shown. '
           'Useful example: Specify "qbase qparent qtip" to hide the '
           'standard tags inserted by the Mercurial Queues Extension. '
           'Default: None (leave blank)')),
@@ -709,12 +714,21 @@ INFO = (
           '<li><b>never</b>: Never show any prompt to activate any bookmarks.'
           '</ul><p>'
           'Default: prompt')),
+    _fi(_('Show Family Line'), 'tortoisehg.showfamilyline', genBoolRBGroup,
+        _('Show indirect revision dependency on the revision graph '
+          'when filtered by revset. Default: True<p>'
+          '<b>Note</b>: Calculating family line may be slow in some cases. '
+          'This option is expected to be removed if the performance issue is '
+          'solved.')),
     )),
-({'name': 'commit', 'label': _('Commit', 'config item'), 'icon': 'menucommit'},
+({'name': 'commit', 'label': _('Commit', 'config item'), 'icon': 'hg-commit'},
  (
     _fi(_('Username'), 'ui.username', genEditCombo,
         _('Name associated with commits.  The common format is:<br>'
           'Full Name &lt;email@example.com&gt;')),
+    _fi(_('Ask Username'), 'ui.askusername', genBoolRBGroup,
+        _('If no username has been specified, the user will be prompted to '
+          'enter a username.  Default: False')),
     _fi(_('Summary Line Length'), 'tortoisehg.summarylen', genIntEditCombo,
         _('Suggested length of commit message lines. A red vertical '
           'line will mark this length.  CTRL-E will reflow the current '
@@ -819,7 +833,7 @@ INFO = (
           'Default: "ssh" or "TortoisePlink.exe -ssh -2" (Windows)')),
     )),
 
-({'name': 'web', 'label': _('Server'), 'icon': 'proxy'}, (
+({'name': 'web', 'label': _('Server'), 'icon': 'hg-serve'}, (
     _fi(_('<b>Repository Details:</b>'), None, genSpacer, ''),
     _fi(_('Name'), 'web.name', genEditCombo,
         _('Repository name to use in the web interface, and by TortoiseHg '
@@ -1009,7 +1023,7 @@ INFO = (
           'You may include groups in issue.regex, and corresponding {n} '
           'tokens in issue.link (where n is a non-negative integer). '
           '{0} refers to the entire string matched by issue.regex, '
-          'while {1} refers to the first group and so on. If no {n} tokens'
+          'while {1} refers to the first group and so on. If no {n} tokens '
           'are found in issue.link, the entire matched string is appended '
           'instead.')),
     _fi(_('Inline Tags'), 'tortoisehg.issue.inlinetags', genBoolRBGroup,
@@ -1113,7 +1127,7 @@ INFO = (
     _fi(_('Servers'), 'projrc.servers', genEditCombo,
         _('List of Servers from which "projrc" configuration files must be '
           'pulled. Set it to "*" to pull from all servers. Set it to "default" '
-          'to pull from the default sync path.'
+          'to pull from the default sync path. '
           'Default is pull from NO servers.')),
     _fi(_('Include'), 'projrc.include', genEditCombo,
         _('List of settings that will be pulled from the project configuration '
@@ -1157,7 +1171,7 @@ class SettingsDialog(QDialog):
         self.setWindowTitle(_('TortoiseHg Settings'))
         self.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint
                             | Qt.WindowMaximizeButtonHint)
-        self.setWindowIcon(qtlib.geticon('settings_repo'))
+        self.setWindowIcon(qtlib.geticon('thg-repoconfig'))
 
         if not hasattr(wconfig.config(), 'write'):
             qtlib.ErrorMsgBox(_('Iniparse package not found'),
@@ -1189,7 +1203,7 @@ class SettingsDialog(QDialog):
         self.conftabs = QTabWidget()
         layout.addWidget(self.conftabs)
         utab = SettingsForm(rcpath=scmutil.userrcpath(), focus=focus)
-        self.conftabs.addTab(utab, qtlib.geticon('settings_user'),
+        self.conftabs.addTab(utab, qtlib.geticon('thg-userconfig'),
                              _("%s's global settings") % username())
         utab.restartRequested.connect(self._pushRestartRequest)
 
@@ -1221,7 +1235,7 @@ class SettingsDialog(QDialog):
 
             reporcpath = os.sep.join([repo.root, '.hg', 'hgrc'])
             rtab = SettingsForm(rcpath=reporcpath, focus=focus)
-            self.conftabs.addTab(rtab, qtlib.geticon('settings_repo'),
+            self.conftabs.addTab(rtab, qtlib.geticon('thg-repoconfig'),
                                  _('%s repository settings')
                                  % repoagent.shortName())
             rtab.restartRequested.connect(self._pushRestartRequest)
@@ -1238,10 +1252,10 @@ class SettingsDialog(QDialog):
         self.conftabs.currentChanged.connect(self._currentFormChanged)
 
     def isDirty(self):
-        return util.any(self.conftabs.widget(i).isDirty()
-                        for i in xrange(self.conftabs.count()))
+        return any(self.conftabs.widget(i).isDirty()
+                   for i in xrange(self.conftabs.count()))
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _pushRestartRequest(self, key):
         self._restartreqs.add(unicode(key))
 
@@ -1254,7 +1268,7 @@ class SettingsDialog(QDialog):
                                'for the following changes to take effect:'),
                              ', '.join(sorted(self._restartreqs)))
             self._restartreqs.clear()
-        return util.all(results)
+        return all(results)
 
     def canExit(self):
         if self.isDirty():
@@ -1303,7 +1317,7 @@ class SettingsDialog(QDialog):
 class SettingsForm(QWidget):
     """Widget for each settings file"""
 
-    restartRequested = pyqtSignal(unicode)
+    restartRequested = pyqtSignal(str)
 
     def __init__(self, rcpath, focus=None, parent=None, readonly=False):
         super(SettingsForm, self).__init__(parent)
@@ -1326,8 +1340,10 @@ class SettingsForm(QWidget):
         self.fnedit = QLineEdit()
         self.fnedit.setReadOnly(True)
         self.fnedit.setFrame(False)
-        self.fnedit.setFocusPolicy(Qt.NoFocus)
-        self.fnedit.setStyleSheet('QLineEdit { background: transparent; }')
+        self.fnedit.setFocusPolicy(Qt.ClickFocus)
+        p = self.fnedit.palette()
+        p.setColor(QPalette.Base, Qt.transparent)
+        self.fnedit.setPalette(p)
         edit = QPushButton(_('Edit File'))
         edit.clicked.connect(self.editClicked)
         self.editbtn = edit
