@@ -6,6 +6,7 @@
 # GNU General Public License version 2, incorporated herein by reference.
 
 import os
+import re
 import sys
 import shlex
 import time
@@ -21,7 +22,7 @@ _encoding = encoding.encoding
 _fallbackencoding = encoding.fallbackencoding
 
 # extensions which can cause problem with TortoiseHg
-_extensions_blacklist = ('color', 'pager', 'progress')
+_extensions_blacklist = ('color', 'pager', 'progress', 'zeroconf')
 
 from tortoisehg.util import paths
 from tortoisehg.util.hgversion import hgversion
@@ -103,6 +104,9 @@ def fromutf(s):
         return str(fromunicode(s.decode('utf-8', 'replace'), 'replace'))
 
 
+def activebookmark(repo):
+    return repo._activebookmark
+
 def namedbranches(repo):
     branchmap = repo.branchmap()
     dead = repo.deadbranches
@@ -134,8 +138,8 @@ def _getfirstrevisionlabel(repo, ctx):
     bookmarks = ctx.bookmarks()
     if ctx in repo.parents():
         # keep bookmark unchanged when updating to current rev
-        if repo._bookmarkcurrent in bookmarks:
-            return repo._bookmarkcurrent
+        if activebookmark(repo) in bookmarks:
+            return activebookmark(repo)
     else:
         # more common switching bookmark, rather than deselecting it
         if bookmarks:
@@ -726,10 +730,10 @@ def escapepath(path):
     else:
         return p
 
-def escaperev(rev):
+def escaperev(rev, default=None):
     """Convert revision number to command-line-safe string"""
     if rev is None:
-        return None
+        return default
     if rev == nullrev:
         return 'null'
     assert rev >= 0
@@ -829,8 +833,37 @@ def buildcmdargs(name, *args, **opts):
             fullargs.append(_stringify(v))
 
     args = [_stringify(v) for v in args if v is not None]
-    if util.any(e.startswith('-') for e in args):
+    if any(e.startswith('-') for e in args):
         fullargs.append('--')
     fullargs.extend(args)
 
     return fullargs
+
+_urlpassre = re.compile(r'^([a-zA-Z0-9+.\-]+://[^:@/]*):[^@/]+@')
+
+def _reprcmdarg(arg):
+    arg = _urlpassre.sub(r'\1:***@', arg)
+    arg = arg.replace('\n', '^M')
+
+    # only for display; no use to construct command string for os.system()
+    if not arg or ' ' in arg or '\\' in arg or '"' in arg:
+        return '"%s"' % arg.replace('"', '\\"')
+    else:
+        return arg
+
+def prettifycmdline(cmdline):
+    r"""Build pretty command-line string for display
+
+    >>> prettifycmdline(['log', 'foo\\bar', '', 'foo bar', 'foo"bar'])
+    'log "foo\\bar" "" "foo bar" "foo\\"bar"'
+    >>> prettifycmdline(['log', '--template', '{node}\n'])
+    'log --template {node}^M'
+
+    mask password in url-like string:
+
+    >>> prettifycmdline(['push', 'http://foo123:bar456@example.org/'])
+    'push http://foo123:***@example.org/'
+    >>> prettifycmdline(['clone', 'svn+http://:bar@example.org:8080/trunk/'])
+    'clone svn+http://:***@example.org:8080/trunk/'
+    """
+    return ' '.join(_reprcmdarg(e) for e in cmdline)
